#include "virge.h"

/*
  Scans a path with AVP, and returns the result
  
  scanpath = path on *local* hard disk where attachments are unpacked
  
  No, you can not scan a file on your remote WinNT based file server in another country using this function.
  
  Example:
  
  scan_avp(scratch_dir);
*/  
int scan_avp(char *scanpath)
{

	int avp;
	int conn;
	int resp = 0;
	int exitcode;
	int readrecv;

	char *response;
	char *tempresp;
	char scan[512];
	time_t nowtime;
	struct sockaddr_un avp_sock;

	memset(&avp_sock, 0, sizeof(avp_sock));
	avp_sock.sun_family=AF_UNIX;
	strcpy(avp_sock.sun_path, "/var/run/AvpCtl\0\0\0");
	
	if ( (avp = socket(AF_UNIX,SOCK_STREAM,0)) <0)
	{
		virge_debug(2, "[ scan_avp() ] failed to create UNIX socket to AVP");
		virge_syslog("FAILURE: Failed to create a socket in scan_avp()");
		return(-1);
	}

	conn = connect(avp, (struct sockaddr *) &avp_sock, sizeof(avp_sock.sun_family)+strlen("/var/run/AvpCtl"));
	if (conn == -1)
	{
		virge_debug(2, "[ scan_avp() ] failed to connect to AVP using connect()");
		virge_syslog("FAILURE: Failed to connect to AVPDaemon in scan_avp()");
		return(-1);
	}

	/* "Craft" the AVP query string... */
	/* It looks like: <0>time:/path */
	time(&nowtime);
	sprintf(scan, "<0>%.15s:%s", ctime(&nowtime)+4, scanpath);
	virge_debug(3, "[ scan_avp() ] string to send to AVP = [%s]", scan);

	/* Send the request - read response */
	/* Taken from AVP examples, of course :) */
 	if ((write(avp, scan, strlen(scan)+1)) == -1)
	{
		virge_debug(2, "[ scan_avp() ] failed to write to AVP");
		virge_syslog("FAILURE: Failed to write to AVPDaemon in scan_avp()");
		return(-1);
	}
	
	/* Here comes the response... */
	virge_debug(3, "[ scan_avp() ] reading response from AVP");

	read(avp, (char *) &resp, 2);
	exitcode = resp & (0xff-0x30);
	virge_debug(3, "[ scan_avp() ] response received and processed [%d]", exitcode);

	/* Get the virus name (if any are found) - C stuff (more or less) taken from AVP samples */
	/* Damnit - something is wrong here, but I'm not sure what (each new recv()'ed line has trailing garbage - that's why we cut it at '\n') - FIXME */
	if (exitcode == 4)
	{
		int multiple_viruses = 0;
		
		/*
		  Damn - Seems sometimes AVP returns wierd data, or I made some mistake that makes it return wierd data.
		  I'll have to double-check every strstr() now... doh...
		*/
		char *strstr_res = 0;
		
		if ((readrecv = read(avp, (char *) &resp, sizeof(unsigned long))) == -1)
			virge_syslog("[ scan_avp() ] failed to read() - [readrecv == -1]");

		response = (char *) malloc(resp+1);
		response[0] = '\0';
		tempresp = response;
		
		while ( (resp > 0) && ((readrecv = recv(avp, response, resp, 0)) != 0) )
		if (readrecv == -1)
		{
			virge_syslog("[ scan_avp() ] failed to recv()");
		}
		else
		{
			if (strstr(response, "infected:"))
			{
				/* Remove the newline */
				if (strchr(response, '\n'))
					*strchr(response, '\n') = '\0';

				/* There is a space at the end of name (at least for me) - remove it */
				if (response[strlen(response)-1] == ' ')
					response[strlen(response)-1] = '\0';

				/* Hope this +10 offset won't change in AVP */
				if (multiple_viruses == 0)
				{
					if ((strstr_res = strstr(response, "infected: ")+10))
						strncat(VIRUS_NAME, strstr_res, V_BUFSIZE-1);
					else
						strncat(VIRUS_NAME, "-error-", V_BUFSIZE-1);

					multiple_viruses++;
				}
				else
				{
					strncat(VIRUS_NAME, " + ", V_BUFSIZE-1);
					
					if ((strstr_res = strstr(response, "infected: ")+10))
						strncat(VIRUS_NAME, strstr_res, V_BUFSIZE-1);
					else
						strncat(VIRUS_NAME, "-error-", V_BUFSIZE-1);
				}
			}
	
			resp -= readrecv;
			tempresp[readrecv] = '\0';
			tempresp += readrecv;
		}

		virge_syslog("[ scan_avp() ] Virus found: [%s]", VIRUS_NAME);
		virge_debug(1, "[ scan_avp() ] Whoa - got a virus [%s]", VIRUS_NAME);

		free(response);
	}

	if ((close(avp)) == -1)
	{
		virge_syslog("FAILURE: Failed to close socket to AVPDaemon in scan_avp()");
		virge_debug(2, "[ scan_avp() ] failed to close socket to AVP");
	}

	virge_debug(1, "[ scan_avp() ] AVP scan finished");
	return(exitcode);
}
