#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <ctype.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <fcntl.h>
#include <time.h>
#include <errno.h>
#include <syslog.h>
#include <dirent.h>
#include <pwd.h>
#include <signal.h>
#include <string.h>
#include <rfc2045.h>
#include <rfc2047.h>
#include <numlib.h>
#include <regex.h>
#include "virge_regex.h"

#define VIRGE_VERSION	"2.03"

/* Linux specific (it seems) */
#ifdef LINUX
  #include <getopt.h>
  /* Why doesn't <bits/fcntl.h> get included by <fcntl.h> ?!!? - FIXME */
  #define O_NOFOLLOW     0400000
#endif

#ifdef SLOWARIS
  #include <strings.h>
  #include <sys/loadavg.h>
  #define O_NOFOLLOW     0400000
#endif

#ifdef HPUX
  #include <strings.h>
  #include <sys/pstat.h>
  #define O_NOFOLLOW     0400000
  /* I don't have right now access to HP-UX 11.00, but this probably won't be needed for it */
  extern int vsnprintf(char *, size_t, const char *, va_list);
  extern int snprintf(char *, size_t, char *,...);
#endif

#define V_BUFSIZE	4096
#define V_LOGMSG	1024
#define V_FILESIZE	256
#define V_ARGSIZE	256
#define V_EMAILSIZE 256
#define V_MSGSIZE	4096
#define V_MSGID 	1024

#define PROC_LOADAVG	"/proc/loadavg"

/* Number of elements in the arrays (for storing usernames, extensions) */
#define CONFIG_MAX 	128

/* Max length of the element in the array */
#define CONFIG_LEN 	128

/* It's faster like this... */
#define CONFIG_CMP(x,y)        (strncasecmp(x,y,strlen(y)))

#define NOTIFY_VIRUS_FOUND	0
#define NOTIFY_BAD_ATTACHMENTS	1

extern int sections;
extern int CHECK_CONFIG_VERBOSE;
extern int REWRITE;
extern int attachCounter;
extern int rfd;

extern int DEBUG;
extern int CMDLINE_DEBUG;
extern int DEBUG_LEVEL;
extern int CMDLINE_DEBUG_LEVEL;
extern int V_SCAN_AVP;
extern int V_SCAN_SOPHIE;
extern int V_SCAN_TROPHIE;
extern int V_DELIVER_LOADAVG;
extern long V_MAIL_SIZE;

char V_SCAN_SOPHIE_SOCKET[V_FILESIZE];
char V_SCAN_TROPHIE_SOCKET[V_FILESIZE];

char V_DIR[V_FILESIZE];
char V_LOGDIR[V_FILESIZE];
char V_TEMPDIR[V_FILESIZE];
char V_ISODIR[V_FILESIZE];
char V_ETCDIR[V_FILESIZE];
char RUNAS_USER[V_ARGSIZE];
char PROCMAIL[V_FILESIZE];
char SENDMAIL[V_FILESIZE];
char ADMIN[V_FILESIZE];
char VIRGE_FROM[V_FILESIZE];

char LOG_VIRGE[V_FILESIZE];
char LOG_ERROR[V_FILESIZE];
char LOG_ACTION[V_FILESIZE];
char LOG_VIRUS[V_FILESIZE];

char V_CONFIG[V_FILESIZE];
char V_EXT[V_FILESIZE];
char V_USERS[V_FILESIZE];

char V_MAILBODY[V_FILESIZE];
char V_PROGNAME[V_ARGSIZE];
char messageID[V_MSGID];

char MAIL_FROM[V_EMAILSIZE];
char RCPT_TO[V_EMAILSIZE];

char REMOVED[CONFIG_MAX][CONFIG_LEN];
extern int removedCount;

char EXTENSIONS[CONFIG_MAX][CONFIG_LEN];
extern int extensionCount;

char ALLOW_USERS[CONFIG_MAX][CONFIG_LEN];
extern int allowUserCount;

char boundary[V_BUFSIZE];
char VIRUS_NAME[V_BUFSIZE];

char VIRGE_DIR_MODE[16];
char ISOLATED_FILE_MODE[16];

extern int NOTIFY_SENDER_VIRUS;
extern int NOTIFY_SENDER_ATTACHMENT;
extern int NOTIFY_USE_HEADER_RCPT;
char headers[V_BUFSIZE];
char NOTIFY_SUBJECT[V_BUFSIZE];
char NOTIFY_DATE[V_BUFSIZE];
char NOTIFY_TO[V_EMAILSIZE];

extern unsigned int VIRGE_TIMEOUT;

/* virge_log.c */
void virge_debug(int debugLevel, char *debugMessage, ...);
int event_log(char *logPath, char *logMessage, ...);
void virge_syslog(char *syslogMessage, ...);

/* virge_file.c */
void copyFile(char *src, char *dst);
void isolate(int type, char *isoComment, char *isoFile, char *isoDir, int isoNotify, char *isoNotifyMsg);
int virge_mkdir(char *newdir);
int check_directory(const char *checkFile);
char *get_date_dir(char *ddir);
char *mkdir_isolate(char *isolateDir);
void check_req_dir(char *reqDir, char *dirType);
void remove_tempdir(char *removeDir);
int file_exists(char *check_if_exists);
void get_headers(char *mailfile);
char *mystrcasestr(char *str_string, char *str_pattern);
int mystrncasecmp(char *str1, char *str2, int n);

/* virge_mail.c */
void virge_procmail(char *mailFile, char *aParam, int direct_delivery);
void virge_sendmail(char *email, char *sendMessage, char *sendSubject);

/* virge_attachments.c */
int check_extension(char *checkExtension);
int check_extension_regex(char *regexCheckExtension);
void check_attachments(struct rfc2045 *p, struct rfc2045id *id, void *ptr);
void rewrite_attachments(struct rfc2045 *p, struct rfc2045id *id, void *ptr);
void removed_attachments(char *removedList);

/* scan_sophie.c */
int sophie_scanfile(char *scanpath);
int sophie_scandir(char *scandir);

/* scan_trophie.c */
int trophie_scanfile(char *scanpath);
int trophie_scandir(char *scandir);

/* scan_avp.c */
int scan_avp(char *scanpath);

/* virge_config.h */
int load_extensions(void);
int load_allowusers(void);
int config_set(char *confLine, char *confEntry, char *configData);
void load_virge_config(void);
void check_current_config(void);
void check_regex(void);
void print_regex_err(char *regex_test, char *regex_err_msg);

/* rfcfuncs.c */
void rfc2045_error(const char *errmsg);
void do_print_structure(struct rfc2045 *p, struct rfc2045id *id, void *ptr);
int decode_to_file(const char *p, size_t n, void *ptr);
char *tempname(const char *tempdir);
void do_print_section(struct rfc2045 *s, FILE *fp);
void notfound(const char *p);
char *get_suitable_filename(struct rfc2045 *r, const char *pfix, int ignore_filename);
void extract_file(struct rfc2045 *p, const char *filename, int argc, char **argv);
void extract_section(struct rfc2045 *top_rfcp, const char *mimesection, const char *extract_filename, int argc, char **argv, void (*extract_func)(struct rfc2045 *, const char *, int, char **));

/* virge_loadavg.c */
int check_loadavg(void);

/* virge_usage.c */
void usage(void);

/* virge_notify.c */
void notify_sender(int notify_type);
